@@ -138,7 +138,7 @@ class RewindableInputFile(object):
 		# getting more data
 		self.reuse = 0
 		# the internal buffer
-		self.buf = b' ' * buffer_size
+		self.buf = b" " * buffer_size
 		# flag indicating a .seek()-based EOF test is in progress
 		self.gzip_hack_pretend_to_be_at_eof = False
 		# avoid attribute look-ups
@@ -154,7 +154,7 @@ class RewindableInputFile(object):
 
 	def next(self):
 		if self.gzip_hack_pretend_to_be_at_eof:
-			return b''
+			return b""
 		if self.reuse:
 			buf = self.buf[-self.reuse:]
 			self.reuse = 0
@@ -166,7 +166,7 @@ class RewindableInputFile(object):
 
 	def read(self, size = None):
 		if self.gzip_hack_pretend_to_be_at_eof:
-			return b''
+			return b""
 		if self.reuse:
 			if self.reuse < 0:
 				buf = self._read(size - self.reuse)
@@ -309,16 +309,33 @@ class SignalsTrap(object):
 		return False
 
 
-def load_fileobj(fileobj, gz = None, xmldoc = None, contenthandler = None):
+# FIXME: remove the following once we drop the ``gz`` keyword argument.
+def _normalize_compress_kwarg(compress = None, gz = False):
+	if gz:
+		warnings.warn(
+			'The gz keyword argument is deprecated. '
+			'Use the compress keyword argument instead.',
+			DeprecationWarning,
+			stacklevel=3)
+		if compress is not None:
+			raise ValueError(
+				'must not specify both compress and gz keyword arguments')
+		compress = 'gz'
+	return compress
+
+
+def load_fileobj(fileobj, compress = None, gz = None, xmldoc = None, contenthandler = None):
 	"""
 	Parse the contents of the file object fileobj, and return the
 	contents as a LIGO Light Weight document tree.  The file object
-	does not need to be seekable.
+	does not need to be seekable.  The file object must be in binary
+	mode.
 
-	If the gz parameter is None (the default) then gzip compressed data
-	will be automatically detected and decompressed, otherwise
-	decompression can be forced on or off by setting gz to True or
-	False respectively.
+	The compress parameter selects the decompression algorithm to use.
+	Valid values are:  "auto" to automatically deduce the decompression
+	scheme from the file format, "gz" to force gzip decompression,
+	False to disable decompression, or None to select the default
+	behaviour (currently equivalent to "auto").
 
 	If the optional xmldoc argument is provided and not None, the
 	parsed XML tree will be appended to that document, otherwise a new
@@ -333,23 +350,55 @@ def load_fileobj(fileobj, gz = None, xmldoc = None, contenthandler = None):
 	>>> xmldoc = load_fileobj(f, contenthandler = ligolw.LIGOLWContentHandler)
 
 	The contenthandler argument specifies the SAX content handler to
-	use when parsing the document.  The contenthandler is a required
-	argument.  See the ligo.lw package documentation for typical
-	parsing scenario involving a custom content handler.  See
-	ligo.lw.ligolw.PartialLIGOLWContentHandler and
+	use when parsing the document.  contenthandler is a required
+	argument.  See the ligo.lw package documentation for an explanation
+	of a typical document parsing scenario and the content handler it
+	uses.  See ligo.lw.ligolw.PartialLIGOLWContentHandler and
 	ligo.lw.ligolw.FilteringLIGOLWContentHandler for examples of custom
 	content handlers used to load subsets of documents into memory.
 	"""
 	if contenthandler is None:
 		raise ValueError("missing required keyword argument \"contenthandler\"")
-	if gz or gz is None:
+
+	# FIXME: remove the following once we drop the ``gz`` keyword argument.
+	compress = _normalize_compress_kwarg(compress = compress, gz = gz)
+
+	if compress is None:
+		# select default behaviour
+		compress = "auto"
+
+	if compress == "auto":
+		# set keyword argument automatically from file format
 		fileobj = RewindableInputFile(fileobj)
 		magic = fileobj.read(2)
 		fileobj.seek(0, os.SEEK_SET)
-		if gz or magic == b'\037\213':
-			fileobj = gzip.GzipFile(mode = "rb", fileobj = fileobj)
+		if magic == b"\037\213":
+			compress = "gz"
+		else:
+			# format not recognized, assume not compressed
+			compress = False
+
+	#
+	# select stream decoder
+	#
+
+	if compress == False:
+		# pass-through
+		pass
+	elif compress == "gz":
+		# gzip decompression
+		fileobj = gzip.GzipFile(mode = "rb", fileobj = fileobj if type(fileobj) == RewindableInputFile else RewindableInputFile(fileobj))
+	else:
+		# oops
+		raise ValueError("unrecognized compress \"%s\"" % compress)
+
+	#
+	# parse stream into XML tree and return it
+	#
+
 	if xmldoc is None:
 		xmldoc = ligolw.Document()
+
 	ligolw.make_parser(contenthandler(xmldoc)).parse(fileobj)
 	return xmldoc
 
@@ -410,37 +459,52 @@ def load_url(url, verbose = False, **kwargs):
 		return load_fileobj(fileobj, **kwargs)
 
 
-def write_fileobj(xmldoc, fileobj, gz = False, compresslevel = 3, **kwargs):
+def write_fileobj(xmldoc, fileobj, compress = None, gz = False, compresslevel = 3, **kwargs):
 	"""
 	Writes the LIGO Light Weight document tree rooted at xmldoc to the
 	given file object.  Internally, the .write() method of the xmldoc
 	object is invoked and any additional keyword arguments are passed
-	to that method.  The file object need not be seekable.  The output
-	data is gzip compressed on the fly if gz is True, and in that case
-	the compresslevel parameter sets the gzip compression level (the
-	default is 3).
+	to that method.  The file object need not be seekable.  The file
+	object must be in binary mode.
+
+	The compress parameter selects the file compression format to use.
+	Valid values are:  False to disable compression, "gz" to select
+	gzip compression, None to select the default behaviour (disable
+	compression)..  When gzip compression is slected, the compresslevel
+	parameter sets the gzip compression level (the default is 3).
 
 	Example:
 
 	>>> import sys
 	>>> from ligo.lw import ligolw
 	>>> xmldoc = load_filename("demo.xml", contenthandler = ligolw.LIGOLWContentHandler)
-	>>> write_fileobj(xmldoc, sys.stdout)	# doctest: +NORMALIZE_WHITESPACE
-	<?xml version='1.0' encoding='utf-8'?>
-	<!DOCTYPE LIGO_LW SYSTEM "http://ldas-sw.ligo.caltech.edu/doc/ligolwAPI/html/ligolw_dtd.txt">
-	<LIGO_LW>
-		<Table Name="demo:table">
-			<Column Type="lstring" Name="name"/>
-			<Column Type="real8" Name="value"/>
-			<Stream Delimiter="," Type="Local" Name="demo:table">
-	"mass",0.5,"velocity",34
-			</Stream>
-		</Table>
-	</LIGO_LW>
+	>>> write_fileobj(xmldoc, open("/dev/null","wb"))
 	"""
+	# FIXME: remove the following once we drop the ``gz`` keyword argument.
+	compress = _normalize_compress_kwarg(compress = compress, gz = gz)
+
+	if compress is None:
+		# select default behaviour
+		compress = False
+
 	with NoCloseFlushWrapper(fileobj) as fileobj:
-		if gz:
+		#
+		# select stream encoder
+		#
+
+		if compress == False:
+			# no compression
+			pass
+		elif compress == "gz":
 			fileobj = gzip.GzipFile(mode = "wb", fileobj = fileobj, compresslevel = compresslevel)
+		else:
+			# oops
+			raise ValueError("unrecognized compress \"%s\"" % compress)
+
+		#
+		# write file
+		#
+
 		with codecs.getwriter("utf_8")(fileobj) as fileobj:
 			xmldoc.write(fileobj, **kwargs)
 
@@ -475,13 +539,22 @@ class tildefile(object):
 		return False
 
 
-def write_filename(xmldoc, filename, verbose = False, gz = False, with_mv = True, trap_signals = SignalsTrap.default_signals, **kwargs):
+def write_filename(xmldoc, filename, verbose = False, compress = None, gz = False, with_mv = True, trap_signals = SignalsTrap.default_signals, **kwargs):
 	"""
 	Writes the LIGO Light Weight document tree rooted at xmldoc to the
 	file name filename.  If filename is None the file is written to
 	stdout, otherwise it is written to the named file.  Friendly
 	verbosity messages are printed while writing the file if verbose is
-	True.  The output data is gzip compressed on the fly if gz is True.
+	True.
+
+	The compress keyword argument selects the compression format to
+	use.  Recognized values are:  "auto" to automatically select the
+	format based on filename, "gz" to force gzip compression, False to
+	disable compression, None to select the default behaviour (which is
+	currently "auto").  When "auto" is the mode selected, if the
+	filename does not match a recognized pattern or filename is None
+	(writing to stdout) then compression is disabled.
+
 	If with_mv is True and filename is not None the filename has a "~"
 	appended to it and the file is written to that name then moved to
 	the requested name once the write has completed successfully.
@@ -504,26 +577,41 @@ def write_filename(xmldoc, filename, verbose = False, gz = False, with_mv = True
 	Example:
 
 	>>> write_filename(xmldoc, "demo.xml")	# doctest: +SKIP
-	>>> write_filename(xmldoc, "demo.xml.gz", gz = True)	# doctest: +SKIP
+	>>> write_filename(xmldoc, "demo.xml.gz", compress = 'gz')	# doctest: +SKIP
 	"""
+	# FIXME: remove the following once we drop the ``gz`` keyword argument.
+	compress = _normalize_compress_kwarg(compress = compress, gz = gz)
+
+	#
+	# select format
+	#
+
+	if compress is None:
+		# select default behaviour
+		compress = "auto"
+	if compress == "auto":
+		if filename is None:
+			# writing to stdout:  no filename = cannot deduce
+			# compression format = turn compression off
+			compress = False
+		elif filename.endswith(".gz"):
+			compress = "gz"
+		else:
+			# filename scheme not recognized, disable
+			# compression
+			compress = False
+
 	if verbose:
 		sys.stderr.write("writing %s ...\n" % (("'%s'" % filename) if filename is not None else "stdout"))
 	with SignalsTrap(trap_signals):
 		if filename is None:
 			# In Python 3, sys.stdout has a .buffer attribute
 			# that is the underyling byte-oriented stream.
-			fileobj = sys.stdout
-			try:
-				fileobj = fileobj.buffer
-			except AttributeError:
-				pass
-			write_fileobj(xmldoc, fileobj, gz = gz, **kwargs)
+			write_fileobj(xmldoc, sys.stdout.buffer if hasattr(sys.stdout, "buffer") else sys.stdout, compress = compress, **kwargs)
 		else:
-			if not gz and filename.endswith(".gz"):
-				warnings.warn("filename '%s' ends in '.gz' but file is not being gzip-compressed" % filename, UserWarning)
-			binary_open = lambda filename: open(filename, 'wb')
+			binary_open = lambda filename: open(filename, "wb")
 			with (binary_open if not with_mv else tildefile)(filename) as fileobj:
-				write_fileobj(xmldoc, fileobj, gz = gz, **kwargs)
+				write_fileobj(xmldoc, fileobj, compress = compress, **kwargs)
 
 
 def write_url(xmldoc, url, **kwargs):
@@ -540,6 +628,6 @@ def write_url(xmldoc, url, **kwargs):
 	Example:
 
 	>>> write_url(xmldoc, "file:///data.xml")	# doctest: +SKIP
-	>>> write_url(xmldoc, "file:///data.xml.gz", gz = True)	# doctest: +SKIP
+	>>> write_url(xmldoc, "file:///data.xml.gz", compress = 'gz')	# doctest: +SKIP
 	"""
 	return write_filename(xmldoc, local_path_from_url(url), **kwargs)
